/**
	ConstructionPreviewer
	

	@author Clonkonaut
*/

local dimension_x, dimension_y, clonk, structure, direction, stick_to, ladder_height, last_n, first_set, fail;
local GFX_StructureOverlay = 1;
local GFX_CombineIconOverlay = 2;

public func GetFlipDescription() { return "$TxtFlipDesc$"; }

func Initialize()
{
	SetProperty("Visibility", VIS_Owner);
}

func Set(id to_construct, object constructing_clonk)
{
	if(to_construct!=Ladder)
	{
		
		SetGraphics(nil, to_construct, GFX_StructureOverlay, GFXOV_MODE_Base);
		SetGraphics(nil, to_construct, 3, GFXOV_MODE_Picture, nil, GFX_BLIT_Wireframe);
	}
	else
	{
		//SetGraphics(nil, to_construct, GFX_StructureOverlay, GFXOV_MODE_Base);
	//	SetGraphics(nil, to_construct, 3, GFXOV_MODE_Picture, nil, GFX_BLIT_Wireframe);
		GFX_StructureOverlay = nil;
	}
	
	dimension_x = to_construct->GetDefWidth();
	dimension_y = to_construct->GetDefHeight();

	clonk = constructing_clonk;
	structure = to_construct;
	direction = DIR_Left;
	first_set=true;
	AdjustPreview();
}

// Positions the preview according to the landscape, coloring it green, yellow or red
func AdjustPreview(bool look_up, bool no_call)
{
	var half_y = dimension_y / 2;
	// Do only if not sticking to another object
	if (!stick_to)
	{
		// Place on material
		var search_dir = 1;
		if (look_up) search_dir = -1;
		var x = 0, y = 0; fail = false;
		while(!(!GBackSolid(x,y + half_y) && GBackSolid(x,y + half_y + 1)))
		{
			y += search_dir;
			if (Abs(y) > dimension_y/2)
			{
				fail = true;
				break;
			}
		}
		if (fail && !no_call)
			return AdjustPreview(!look_up, true);
		if (fail)
			return SetClrModulation(RGBa(255,50,50, 100), GFX_StructureOverlay);
		SetPosition(GetX(), GetY() + y);
	}
	if (!CheckConstructionSite(structure, 0, half_y))
		fail = true;
	
	if(structure==Ladder)
	{
		if(!PathFree(GetX(), (GetY()+half_y)-ladder_height, GetX(), GetY()+half_y) || FindObject(Find_Category(C4D_Structure), Find_Or(Find_AtPoint(dimension_x/3),Find_AtPoint(dimension_x/-3))) || FindObject(Find_ID(Ladder), Find_Or(Find_AtPoint(dimension_x/3),Find_AtPoint(dimension_x/-3))))
		fail=true;
		else fail = false;
		
			var n = ladder_height/36;
			if(n>=1 && n != last_n)
			{
				for(var m=1; m<10 ;m++) DetachMesh(m);
					
				for(var j=0; j<n ;j++) AttachMesh(Ladder, "Bone", "Bone", Trans_Translate(0, -18000+36000*(j), 0));
				
				last_n = n;
			}


	}
	
	// intersection-check with all other construction sites... bah
	for(var other_site in FindObjects(Find_ID(ConstructionSite)))
	{
		if(!(other_site->GetLeftEdge()   > GetX()+dimension_x/2  ||
		     other_site->GetRightEdge()  < GetX()-dimension_x/2  ||
		     other_site->GetTopEdge()    > GetY()+half_y  ||
		     other_site->GetBottomEdge() < GetY()-half_y))
			{
				fail = true;
			} 
	}
	
	
	if(!fail)
	{
		if (!stick_to)
			SetClrModulation(RGBa(50,255,50, 100), GFX_StructureOverlay);
		else
			SetClrModulation(RGBa(255,255,50, 200), GFX_StructureOverlay);
	}
	else
		SetClrModulation(RGBa(255,50,50, 100), GFX_StructureOverlay);
		
}

// Positions the preview according to the mouse cursor, calls AdjustPreview afterwards
// x and y are refined mouse coordinates so always centered at the clonk
func Reposition(int x, int y)
{
	if(structure==Ladder)
	{
		var mouse_y = clonk->GetY()+y;
		var ladder_y = GetY()+dimension_y/2;
		var n = ((ladder_y-mouse_y)+18)/36;
		ladder_height = BoundBy(n*36, 36, 252);
	}


	x = BoundBy(x, -dimension_x/2, dimension_x/2);
	y = BoundBy(y, -dimension_y/2, dimension_y/2);
	var found = false;
	if (structure->~ConstructionCombineWith())
	{
		var other = FindObject(Find_Func(structure->ConstructionCombineWith(), this),
		                              Find_InRect(AbsX(clonk->GetX() + x - dimension_x/2 - 10), AbsY(clonk->GetY() + y - dimension_y/2 - 10), dimension_x + 20, dimension_y + 36),
		                              Find_OCF(OCF_Fullcon),
		                              Find_Layer(clonk->GetObjectLayer()),
		                              Find_Allied(clonk->GetOwner()),
		                              Find_NoContainer());
		if(other)
		{
		if (other->GetID() != Ladder)
		{
			if (other->GetX() < GetX())
				x = other->GetX() + other->GetObjWidth()/2 + dimension_x / 2;
			else
				x = other->GetX() - other->GetObjWidth()/2 - dimension_x / 2;
			y = other->GetY();
			stick_to = other;
			found = true;
		}
		else
		{
			if (other->GetY() > GetY())
				y = other->GetY() - other->GetObjHeight()/2 - dimension_y / 2;
			x = other->GetX()+1;
			stick_to = other;
			found = true;
		}
	}
	}
	if (!found)
	{
		x = clonk->GetX() + x;
		y = clonk->GetY() + y;
	}

	if (!found && stick_to)
	{
		stick_to = nil;
		SetGraphics(nil, nil, GFX_CombineIconOverlay);
	} else if (stick_to) {
		SetGraphics(nil, ConstructionPreviewer_IconCombine, GFX_CombineIconOverlay, GFXOV_MODE_Base);
		var dir = 1;
		var half_ladder = 0;
		if (stick_to->GetX() < GetX()) dir = -1;
		if (structure == Ladder){ dir = 0; half_ladder = 18;}
		SetObjDrawTransform(1000, 0, dimension_x/2 * 1000 * dir, 0, 1000, 0+half_ladder*1000, GFX_CombineIconOverlay);
		Log("hald fldder = %d", half_ladder);
	}

	SetPosition(x, y);
	AdjustPreview();
}

// Flips the preview horizontally
func Flip()
{
	// Flip not allowed?
	if (structure->~NoConstructionFlip()) return;

	if (direction == DIR_Left)
	{
		direction = DIR_Right;
		SetObjDrawTransform(-1000,0,0, 0,1000,0, GFX_StructureOverlay);
	} else {
		direction = DIR_Left;
		SetObjDrawTransform(1000,0,0, 0,1000,0, GFX_StructureOverlay);
	}
}